#ifndef LOGIC_H
#define LOGIC_H

/**
 * @file
 * Klassendeklaration fuer die Logikklasse
 *
 * @author
 * Philip Lehmann-Boehm (minf8180)
 */

#include <QString>
#include <QtNetwork>

#include "types.h"

class Logic : public QObject {
    Q_OBJECT

	private:
		// Die Pause in ms, die der Client bis zum naechsten Serverrequest wartet
		static const float serverRequestPause = 40.0f / 1000.0f;

		// Der Bytecode fuer die Personenabfrage
		static const quint8 cmdGetPersons = 2;

		// Der Timeout in ms, wie lange der Client versucht, sich mit dem Server zu verbinden
		static const int connectTimeout = 1000;

		// Die Personen als statisches Array verwalten.
		Person persons[10000];

		// Die Anzahl der Personen
		int personCount;

		// Die vergangene Zeit seit dem letzten Serverrequest
		float timeSinceLastRequest;

		// Block-Buffer fuer den out-Stream
		QByteArray block;

		// Die Streams zur Serverkommunikation
		QDataStream * out, * in;

		// Die Groesse der ankommenden Daten
		quint16 blockSize;

		// Der Socket zum Server
		QTcpSocket socket;

		// Flag, ob die Daten zum ersten mal vom Server ausgelesen wurden
		bool firstRead;

		// Die aktuelle Rotation
		double rotation;

		/**
		 * Normalisiert einen Winkel in den Grenzen 0-360 Grad
		 * @param angle Der zu normalisierende Winkel
		 */
		void normalizeAngle(double *angle);

	private slots:

		/**
		 * Liest die eingehenden Serverdaten aus
		 */
		void readData();

	public:

		/**
		 * Konstruktor
		 */
		Logic();

		/**
		 * Destruktor
		 */
		~Logic();

		/**
		 * Berechnet den naechsten Schritt
		 * @param interval Vergangene Zeit seit dem letzten Schritt
		 * @return Die Schrittberechnung hat geklappt
		 */
		bool nextStep(double interval);

		/**
		 * Liefert den momentanen Stand der Rotation
		 * @return Der momentane Stand der Rotation
		 */
		double getRotation();

		/**
		 * Baut die Verbindung zum Server auf.
		 * @param host Die Serveradresse
		 * @param port Der Verbindungsport
		 * @return Ob die Verbindung geklappt hat oder nicht
		 */
		bool connectToHost(QString host, int port);

		/**
		 * Liefert die Anzahl der Personen
		 * @return Die Anzahl der Personen
		 */
		int getPersonCount();

		/**
		 * Liefert eine bestimmte Person
		 * @param i Der Index der gewuenschten Person
		 * @return Die gewuenschte Person
		 */
		Person* getPerson(int i);
};

#endif
