#ifndef SETTINGS_H
#define SETTINGS_H

/**
 * @file
 * Klassendeklaration fuer die Einstellungen
 *
 * @author
 * Philip Lehmann-Boehm (minf8180)
 */

#include <QtXml>
#include <QString>

class Settings {
	private:
		// Der DOM-Baum der Einstellungen
		QDomDocument settings;

		// Der Dateiname der Einstellungen
		QString filename;

	public:

		/**
		 * Konstruktor
		 * @param filename Der Dateiname der Einstellungen
		 */
		Settings(QString filename);

		/**
		 * Liefert eine Int-Einstellung
		 * @param name Der Name der Einstellung
		 * @param defValue Default-Wert, wenn die Einstellung nicht in der Datei ist
		 * @return Die Einstellung
		 */
		int get(QString setting, int defValue);

		/**
		 * Liefert eine String-Einstellung
		 * @param name Der Name der Einstellung
		 * @param defValue Default-Wert, wenn die Einstellung nicht in der Datei ist
		 * @return Die Einstellung
		 */
		QString get(QString setting, QString defValue);

		/**
		 * Liefert eine Float-Einstellung
		 * @param name Der Name der Einstellung
		 * @param defValue Default-Wert, wenn die Einstellung nicht in der Datei ist
		 * @return Die Einstellung
		 */
		float get(QString name, float defValue);

		/**
		 * Aendert eine String-Einstellung
		 * @param name Der Name der Einstellung
		 * @param value Der neue Wert der Einstellung
		 */
		void change(QString name, QString value);

		/**
		 * Aendert eine Int-Einstellung
		 * @param name Der Name der Einstellung
		 * @param value Der neue Wert der Einstellung
		 */
		void change(QString name, int value);

		/**
		 * Aendert eine Float-Einstellung
		 * @param name Der Name der Einstellung
		 * @param value Der neue Wert der Einstellung
		 */
		void change(QString name, float value);

		/**
		 * Schreibt die Einstellungen zurueck in die Datei
		 */
		void write();
};

// Fehlerklasse, die geworfen wird, falls etwas mit den Einstellungen schiefgeht.
class SettingsError {};

#endif
